@@ -18,19 +18,15 @@
 
 /* exported PlacesBookmarksManager PlacesRecentManager PlacesTrashManager PlacesVolumesManager */
 
-const Gettext = imports.gettext;
-const Signals = imports.signals;
-
 const { Gio, GLib, Gtk, Shell } = imports.gi;
 
 const ExtensionUtils = imports.misc.extensionUtils;
+const { EventEmitter } = imports.misc.signals;
 
 const Main = imports.ui.main;
 const { ShellMountOperation } = imports.ui.shellMountOperation;
 
-const Me = ExtensionUtils.getCurrentExtension();
-
-const _ = Gettext.domain(Me.metadata['gettext-domain']).gettext;
+const _ = ExtensionUtils.gettext;
 
 class PlaceInfo {
     constructor(...params) {
@@ -287,9 +283,10 @@ class PlaceTrashItemInfo extends PlaceInfo {
     }
 }
 
-class PlacesManager {
-    constructor(...params) {
-        this._init(...params);
+class PlacesManager extends EventEmitter {
+    constructor() {
+        super();
+        this._init();
     }
 
     _init() {
@@ -331,7 +328,6 @@ class PlacesManager {
         return this._places[kind];
     }
 }
-Signals.addSignalMethods(PlacesManager.prototype);
 
 var PlacesBookmarksManager = class extends PlacesManager {
     _init() {
