@@ -16,17 +16,13 @@
  * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
-/* exported init buildPrefsWidget */
+/* exported init fillPreferencesWindow */
 
-const Gettext = imports.gettext;
-
-const { Gio, GLib, GObject, Gtk } = imports.gi;
+const { Adw, Gio, GLib, GObject, Gtk } = imports.gi;
 
 const ExtensionUtils = imports.misc.extensionUtils;
 
-const Me = ExtensionUtils.getCurrentExtension();
-
-const _ = Gettext.domain(Me.metadata['gettext-domain']).gettext;
+const _ = ExtensionUtils.gettext;
 
 function checkAppId(appId) {
     if (appId.indexOf('.desktop') < 0) return false;
@@ -40,66 +36,21 @@ function checkAppId(appId) {
 function getPreferences(group) {
     let prefGroup = null;
     switch (group) {
-    case 'generally':
+    case 'activities':
         prefGroup = {
-            title: _('Generally'),
+            title: _('Activities'),
             prefs: [
-                {
-                    type: 'switcher',
-                    key: 'enable-hot-corners',
-                    label: _('Enable hot corners'),
-                    tooltip: _('If true, the activities overview can be accessed by moving the mouse to the top-left corner.'),
-                    schema: 'org.gnome.desktop.interface',
-                },
                 {
                     type: 'combo',
                     key: 'show-activities',
-                    label: _('Activities button'),
-                    tooltip: _('Select the display mode for the Activities button in the top bar.'),
+                    title: _('Activities button'),
+                    subtitle: _('Select the desired display mode of Activities button in the top bar.'),
                     values: {
                         'default': _('Default'),
                         'icon': _('GNOME icon'),
                         'none': _('Not show'),
                     },
                 },
-                {
-                    type: 'switcher',
-                    key: 'always-show-universal-access-status',
-                    label: _('Always show the Universal Access status icon'),
-                    tooltip: _('This key overrides the automatic hiding of the Universal Access status icon when no accessibility features are enabled.'),
-                    schema: 'org.gnome.desktop.a11y',
-                },
-                {
-                    type: 'switcher',
-                    key: 'always-show-input-sources-status',
-                    label: _('Always show the Input Sources status icon'),
-                    tooltip: _('This key overrides the automatic hiding of the Input Sources status icon when when only one Input Sources are enabled.'),
-                },
-                {
-                    type: 'switcher',
-                    key: 'show-dropdown-arrow',
-                    label: _('Dropdown arrow'),
-                    tooltip: _('Determines whether or not buttons appear in the top bar with a dropdown arrow.'),
-                },
-                {
-                    type: 'separator',
-                },
-                {
-                    type: 'switcher',
-                    key: 'show-dash',
-                    label: _('Dash'),
-                    tooltip: _('Determines whether the Dash is displayed or not.'),
-                },
-                {
-                    type: 'separator',
-                },
-                {
-                    type: 'switcher',
-                    key: 'center-new-windows',
-                    label: _('Place new windows in the center'),
-                    tooltip: _('When true, the new windows will always be put in the center of the active screen of the monitor.'),
-                    schema: 'org.gnome.mutter',
-                },
             ],
         };
         break;
@@ -110,8 +61,8 @@ function getPreferences(group) {
                 {
                     type: 'combo',
                     key: 'show-applications',
-                    label: _('Applications button'),
-                    tooltip: _('Select the display mode for the Applications button in the top bar.'),
+                    title: _('Applications button'),
+                    subtitle: _('Select the display mode for the Applications button in the top bar.'),
                     values: {
                         'all' : _('Icon and Text'),
                         'icon': _('Icon'),
@@ -120,23 +71,15 @@ function getPreferences(group) {
                 },
                 {
                     type: 'label',
-                    label: _('To specify which applications will be displayed the GNOME menu editor (alacarte) is required.'),
-                },
-                {
-                    type: 'separator',
-                },
-                {
-                    type: 'switcher',
-                    key: 'show-dash-applications',
-                    label: _('Show Applications icon on Dash'),
-                    tooltip: _('Determines whether the Show Applications icon in the Dash is displayed or not.'),
+                    title: _('Change which applications are shown'),
+                    subtitle: _('To specify which applications will be displayed the GNOME menu editor (alacarte) is required.'),
                 },
             ],
         };
         if (GLib.file_test('/usr/bin/alacarte', GLib.FileTest.EXISTS)) prefGroup.prefs.splice(1, 1, {
             type : 'button',
-            label: _('Change which applications are shown'),
-            tooltip: _('Use the GNOME menu editor to specify which applications will be displayed.'),
+            title: _('Change which applications are shown'),
+            subtitle: _('Use the GNOME menu editor to specify which applications will be displayed.'),
             app: 'alacarte',
             appName: _('Main Menu'),
         });
@@ -148,8 +91,8 @@ function getPreferences(group) {
                 {
                     type: 'combo',
                     key: 'show-places',
-                    label: _('Places button'),
-                    tooltip: _('Select the display mode for the Places button in the top bar.'),
+                    title: _('Places button'),
+                    subtitle: _('Select the display mode for the Places button in the top bar.'),
                     values: {
                         'all': _('Icon and Text'),
                         'icon': _('Icon'),
@@ -159,23 +102,13 @@ function getPreferences(group) {
                 {
                     type: 'combo',
                     key: 'bookmarks-position',
-                    label: _('Position of Bookmarks'),
-                    tooltip: _('Select the position for the bookmarks in the places menu. Custom bookmarks are displayed in a submenu.'),
+                    title: _('Position of Bookmarks'),
+                    subtitle: _('Select the position for the bookmarks in the places menu. Custom bookmarks are displayed in a submenu.'),
                     values: {
                          'mainmenu': _('Main Menu'),
                          'submenu': _('Submenu'),
                     },
                 },
-                {
-                    type: 'separator',
-                },
-                {
-                    type: 'switcher',
-                    key: 'show-desktop-icons',
-                    label: _('Have file manager handle the desktop'),
-                    tooltip: _('If set to true, then file manager will draw the icons on the desktop.'),
-                    schema: 'org.gnome.desktop.background',
-                },
             ],
         };
         break;
@@ -186,8 +119,8 @@ function getPreferences(group) {
                 {
                     type: 'combo',
                     key: 'show-favorites',
-                    label: _('Favorites button'),
-                    tooltip: _('Select the display mode for the Favorites button in the top bar.'),
+                    title: _('Favorites button'),
+                    subtitle: _('Select the display mode for the Favorites button in the top bar.'),
                     values: {
                         'all': _('Icon and Text'),
                         'icon': _('Icon'),
@@ -197,8 +130,8 @@ function getPreferences(group) {
                 {
                     type: 'combo',
                     key: 'favorites-scope',
-                    label: _('Menu of the Favorites button'),
-                    tooltip: _('Change which favorites are shown in the Favorites button menu.'),
+                    title: _('Menu of the Favorites button'),
+                    subtitle: _('Change which favorites are shown in the Favorites button menu.'),
                     values: {
                         'own': _('Only own Favorites'),
                         'all': _('All Favorites'),
@@ -207,7 +140,7 @@ function getPreferences(group) {
                 {
                     type: 'favorites',
                     key: 'favorite-apps',
-                    label: _('Own Favorites'),
+                    title: _('Own Favorites'),
                 },
             ],
         };
@@ -219,39 +152,8 @@ function getPreferences(group) {
                 {
                     type: 'switcher',
                     key: 'show-events-icon',
-                    label: _('Show day events icon'),
-                    tooltip: _('If true, display day events icon, in addition to time.'),
-                },
-                {
-                    type: 'switcher',
-                    key: 'clock-show-date',
-                    label: _('Show date in clock'),
-                    tooltip: _('If true, display date in the clock, in addition to time.'),
-                    schema: 'org.gnome.desktop.interface',
-                },
-                {
-                    type: 'switcher',
-                    key: 'clock-show-seconds',
-                    label: _('Whether the clock shows seconds'),
-                    tooltip: _('If true, display seconds in the clock.'),
-                    schema: 'org.gnome.desktop.interface',
-                },
-                {
-                    type: 'switcher',
-                    key: 'clock-show-weekday',
-                    label: _('Show weekday in clock'),
-                    tooltip: _('If true, display weekday in the clock, in addition to time.'),
-                    schema: 'org.gnome.desktop.interface',
-                },
-                {
-                    type: 'separator',
-                },
-                {
-                    type: 'switcher',
-                    key: 'show-weekdate',
-                    label: _('Show the week date in the calendar'),
-                    tooltip: _('If true, display the ISO week date in the calendar.'),
-                    schema: 'org.gnome.desktop.calendar',
+                    title: _('Show day events icon'),
+                    subtitle: _('If true, display day events icon, in addition to time.'),
                 },
             ],
         };
@@ -263,40 +165,55 @@ function getPreferences(group) {
                 {
                     type: 'combo',
                     key: 'show-systemmenu',
-                    label: _('System menu button'),
-                    tooltip: _('Select the display mode for the System menu button in the top bar.'),
+                    title: _('System menu button'),
+                    subtitle: _('Select the display mode for the System menu button in the top bar.'),
                     values: {
                         'default': _('Default'),
-                        'icon': _('Shutdown icon')
-                    }
+                        'icon': _('Shutdown icon'),
+                    },
                 },
                 {
                     type: 'switcher',
-                    key: 'show-battery-percentage',
-                    label: _('Show battery percentage'),
-                    tooltip: _('If true, display battery percentage in the status menu, in addition to the icon.'),
-                    schema: 'org.gnome.desktop.interface',
-                },
-                {
-                    type: 'separator',
-                },
-                {
-                    type: 'switcher',
-                    key: 'always-show-audio-input',
-                    label: _('Always show the Audio Input widget'),
-                    tooltip: _('This key overrides the automatic hiding of the Audio Input widget when none application is recording audio.'),
+                    key: 'show-gnome-terminal',
+                    title: _('Show GNOME Terminal'),
+                    subtitle: _('Specifies whether or not to show GNOME Terminal in the system menu.'),
                 },
                 {
                     type: 'switcher',
                     key: 'show-hybrid-sleep',
-                    label: _('Show Hybrid Sleep'),
-                    tooltip: _('If true, display Hybrid Sleep item in the Shutdown menu.'),
+                    title: _('Show Hybrid Sleep'),
+                    subtitle: _('If true, display Hybrid Sleep item in the Shutdown menu.'),
                 },
                 {
                     type: 'switcher',
                     key: 'show-hibernate',
-                    label: _('Show Hibernate'),
-                    tooltip: _('If true, display Hibernate item in the Shutdown menu.'),
+                    title: _('Show Hibernate'),
+                    subtitle: _('If true, display Hibernate item in the Shutdown menu.'),
+                },
+                {
+                    type: 'switcher',
+                    key: 'show-basic-settings',
+                    title: _('Show Basic Settings'),
+                    subtitle: _('Determines whether or not Basic Settings on System menu is displayed.'),
+                },
+            ],
+        };
+        break;
+    case 'dash':
+        prefGroup = {
+            title: _('Dash'),
+            prefs: [
+                {
+                    type: 'switcher',
+                    key: 'show-dash',
+                    title: _('Dash'),
+                    subtitle: _('Determines whether the Dash is displayed or not.'),
+                },
+                {
+                    type: 'switcher',
+                    key: 'show-dash-applications',
+                    title: _('Show Applications icon on Dash'),
+                    subtitle: _('Determines whether the Show Applications icon in the Dash is displayed or not.'),
                 },
             ],
         };
@@ -307,87 +224,115 @@ function getPreferences(group) {
 }
 
 const PanelManagerPreferences = GObject.registerClass(
-class PanelManagerPreferences extends Gtk.Notebook {
+class PanelManagerPreferences extends Adw.PreferencesPage {
     _init() {
-        super._init({
-            scrollable: true,
-            margin: 15,
-            width_request: 720,
-            height_request: 480,
-        });
+        super._init();
         this._settings = ExtensionUtils.getSettings();
-        this._createPrefGroup(getPreferences('generally'));
+        this._createPrefGroup(getPreferences('activities'));
         this._createPrefGroup(getPreferences('applications'));
         this._createPrefGroup(getPreferences('places'));
         this._createPrefGroup(getPreferences('favorites'));
         this._createPrefGroup(getPreferences('datemenu'));
         this._createPrefGroup(getPreferences('systemmenu'));
-        this.show_all();
+        this._createPrefGroup(getPreferences('dash'));
     }
 
     _createPrefGroup(group) {
-        let prefGroup = new Gtk.Box({
-            orientation: Gtk.Orientation.VERTICAL,
-            margin: 15,
-            spacing: 15,
-        });
+        if (group === null) return;
+        let prefGroup = new Adw.PreferencesGroup({ title: group.title });
         group.prefs.forEach(pref => {
             let prefWidget = this._createPrefWidget(pref);
-            if (prefWidget) prefGroup.add(prefWidget);
+            if (prefWidget !== null) prefGroup.add(prefWidget);
         });
-        this.append_page(prefGroup, new Gtk.Label({ label: group.title }));
+        this.add(prefGroup);
     }
 
     _createPrefWidget(pref) {
+        let prefAction = null;
         let prefWidget = null;
         let settings = pref.schema ? new Gio.Settings({ schema_id: pref.schema }) : this._settings;
         switch (pref.type) {
+        case 'button':
+            prefAction = new Gtk.Button({
+                label: pref.appName,
+                valign: Gtk.Align.CENTER,
+                width_request: 240,
+            });
+            prefAction.connect('clicked', () => {
+                let [success, argv] = GLib.shell_parse_argv(pref.app);
+                GLib.spawn_async(null, argv, null, GLib.SpawnFlags.SEARCH_PATH, null);
+            });
+            break;
+        case 'combo':
+            let model = new Gtk.StringList();
+            for (let value in pref.values) {
+                model.append(pref.values[value]);
+            }
+            prefAction = new Gtk.DropDown({
+                model: model,
+                valign: Gtk.Align.CENTER,
+                width_request: 240,
+            });
+            prefAction.connect('notify::selected-item', () => {
+                settings.set_enum(pref.key, prefAction.get_selected());
+            });
+            prefAction.set_selected(settings.get_enum(pref.key));
+            settings.connect('changed::%s'.format(pref.key), () => {
+                prefAction.set_selected(settings.get_enum(pref.key));
+            });
+            break;
         case 'favorites':
-            prefWidget = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL });
+            let child = new Gtk.Box({
+                orientation: Gtk.Orientation.VERTICAL,
+                margin_top: 12,
+                margin_bottom: 12,
+                margin_start: 12,
+                margin_end: 12,
+                spacing: 8,
+            });
+            prefWidget = new Adw.PreferencesRow({ child: child }); 
             let frame = new Gtk.Frame();
-            prefWidget.add(frame);
-            let scrolledWindow = new Gtk.ScrolledWindow({ expand: true });
+            child.append(frame);
+            let scrolledWindow = new Gtk.ScrolledWindow();
+            scrolledWindow.set_min_content_height(200);
             scrolledWindow.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
-            frame.add(scrolledWindow);
+            frame.set_child(scrolledWindow);
             let listStore = new Gtk.ListStore();
             listStore.set_column_types([ GObject.TYPE_STRING, GObject.TYPE_STRING ]);
             let treeView = new Gtk.TreeView({ model: listStore });
-            scrolledWindow.add(treeView);
+            scrolledWindow.set_child(treeView);
             treeView.get_selection().set_mode(Gtk.SelectionMode.SINGLE);
             let columnFavorites = new Gtk.TreeViewColumn({
-                title: pref.label,
-                expand: true,
+                title: pref.title,
                 sort_column_id: 1,
             });
             let textRenderer = new Gtk.CellRendererText();
             columnFavorites.pack_start(textRenderer, true);
             columnFavorites.add_attribute(textRenderer, 'text', 1);
             treeView.insert_column(columnFavorites, 0);
-            let toolbar = new Gtk.Toolbar();
-            toolbar.get_style_context().add_class(Gtk.STYLE_CLASS_INLINE_TOOLBAR);
-            prefWidget.add(toolbar);
-            let addButton = new Gtk.ToolButton({
-                icon_name: 'list-add',
+            let toolbar = new Gtk.CenterBox({ orientation: Gtk.Orientation.HORIZONTAL });
+            child.append(toolbar);
+            let addButton = new Gtk.Button({
                 label: _('Select application'),
-                is_important: true,
+                hexpand: true,
             });
             addButton.connect('clicked', () => {
                 this._favoriteSearch(settings, pref.key);
             });
-            toolbar.add(addButton);
-            let inputButton = new Gtk.ToolButton({
-                icon_name: 'list-add',
+            toolbar.set_start_widget(addButton);
+            let inputButton = new Gtk.Button({
                 label: _('Desktop file ID'),
-                is_important: true,
+                hexpand: true,
+                margin_start: 8,
+                margin_end: 8,
             });
             inputButton.connect('clicked', () => {
                 this._favoriteInput(settings, pref.key);
             });
-            toolbar.add(inputButton);
-            let delButton = new Gtk.ToolButton({
-                icon_name: 'list-remove',
+            toolbar.set_center_widget(inputButton);
+            let delButton = new Gtk.Button({
                 label: _('Remove Favorite'),
-                is_important: true,
+                hexpand: true,
             });
             delButton.connect('clicked', () => {
                 let [ any, model, iter ] = treeView.get_selection().get_selected();
@@ -402,7 +347,7 @@ class PanelManagerPreferences extends Gtk.Notebook {
                     settings.set_strv(pref.key, strArray);
                 }
             });
-            toolbar.add(delButton);
+            toolbar.set_end_widget(delButton);
             let strArray = [];
             let treeViewSelection = treeView.get_selection();
             let updateListStore = () => {
@@ -425,63 +370,27 @@ class PanelManagerPreferences extends Gtk.Notebook {
             updateListStore();
             break;
         case 'label':
-            prefWidget = new Gtk.Label({
-                label: pref.label,
-                xalign: 0,
+            prefWidget = new Adw.ActionRow({
+                title: pref.title,
+                subtitle: pref?.subtitle ?? null,
             });
             break;
-        case 'separator':
-            prefWidget = new Gtk.Separator();
+        case 'switcher':
+            prefAction = new Gtk.Switch({
+                active: settings.get_boolean(pref.key),
+                valign: Gtk.Align.CENTER,
+            });
+            settings.bind(pref.key, prefAction, 'active', Gio.SettingsBindFlags.DEFAULT);
             break;
-        default:
-            let prefAction = null;
-            switch (pref.type) {
-            case 'button':
-                prefAction = new Gtk.Button({
-                    label: pref.appName,
-                    valign: Gtk.Align.CENTER,
-                    width_request: 240,
-                });
-                prefAction.connect('clicked', () => {
-                    let [success, argv] = GLib.shell_parse_argv(pref.app);
-                    GLib.spawn_async(null, argv, null, GLib.SpawnFlags.SEARCH_PATH, null);
-                });
-                break;
-            case 'combo':
-                prefAction = new Gtk.ComboBoxText({
-                    valign: Gtk.Align.CENTER,
-                    width_request: 240,
-                });
-                for (let value in pref.values) {
-                    prefAction.append(value, pref.values[value]);
-                }
-                prefAction.set_active_id(settings.get_string(pref.key));
-                prefAction.connect('changed', () => {
-                    settings.set_string(pref.key, prefAction.get_active_id());
-                });
-                settings.connect('changed::%s'.format(pref.key), () => {
-                    prefAction.set_active_id(settings.get_string(pref.key));
-                });
-                break;
-            case 'switcher':
-                prefAction = new Gtk.Switch({
-                    active: settings.get_boolean(pref.key),
-                    valign: Gtk.Align.CENTER,
-                });
-                settings.bind(pref.key, prefAction, 'active', Gio.SettingsBindFlags.DEFAULT);
-                break;
-            default: // Nothing to do
-            }
-            if (prefAction) {
-                if (pref.tooltip) prefAction.set_tooltip_text(pref.tooltip);
-                prefWidget = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL });
-                prefWidget.add(new Gtk.Label({
-                    label: pref.label,
-                    hexpand: true,
-                    xalign: 0,
-                }));
-                prefWidget.add(prefAction);
-            }
+        default: // Nothing to do
+        }
+        if (prefAction !== null) {
+            prefWidget = new Adw.ActionRow({
+                title: pref.title,
+                subtitle: pref?.subtitle ?? null,
+            });
+            prefWidget.add_suffix(prefAction);
+            prefWidget.set_activatable_widget(prefAction);
         }
         return prefWidget;
     }
@@ -491,7 +400,7 @@ class PanelManagerPreferences extends Gtk.Notebook {
             title: _('Add Favorite'),
             modal: true,
             resizable: false,
-            transient_for: this.get_toplevel(),
+            transient_for: this.get_root(),
             use_header_bar: true,
         });
         dialog.add_button(_('Cancel'), Gtk.ResponseType.CANCEL);
@@ -505,16 +414,19 @@ class PanelManagerPreferences extends Gtk.Notebook {
         dialog.set_default_response(Gtk.ResponseType.OK);
         let grid = new Gtk.Grid({
             column_spacing: 15,
-            margin: 15,
+            margin_top: 15,
+            margin_bottom: 15,
+            margin_start: 15,
+            margin_end: 15,
         });
-        dialog.get_content_area().add(grid);
+        dialog.get_content_area().append(grid);
         grid.attach(new Gtk.Label({
             label: _('Desktop file ID:'),
             xalign: 0,
         }), 0, 0, 1, 1);
         let entry = new Gtk.Entry({
             activates_default: true,
-            expand: true,
+            hexpand: true,
         });
         entry.connect('changed', () => {
             addButton.sensitive = 0;
@@ -535,7 +447,7 @@ class PanelManagerPreferences extends Gtk.Notebook {
             }
             dialog.destroy();
         });
-        dialog.show_all();
+        dialog.show();
     }
 
     _favoriteSearch(schema, key) {
@@ -543,8 +455,13 @@ class PanelManagerPreferences extends Gtk.Notebook {
         let addButton = dialog.add_button(_('Add'), Gtk.ResponseType.OK);
         addButton.sensitive = 0;
         dialog.set_default_response(Gtk.ResponseType.OK);
-        let grid = new Gtk.Grid({ margin: 15 });
-        dialog.get_content_area().add(grid);
+        let grid = new Gtk.Grid({
+            margin_top: 15,
+            margin_bottom: 15,
+            margin_start: 15,
+            margin_end: 15,
+        });
+        dialog.get_content_area().append(grid);
         let appChooser = new Gtk.AppChooserWidget();
         appChooser.set_default_text('');
         appChooser.set_show_all(true);
@@ -556,6 +473,7 @@ class PanelManagerPreferences extends Gtk.Notebook {
             if (strArray.indexOf(appInfo.get_id()) >= 0) return;
             addButton.sensitive = 1;
         });
+        appChooser.set_size_request(400, 240);
         grid.attach(appChooser, 0, 0, 1, 1);
         dialog.connect('response', (dialog, response_id) => {
             if (response_id === Gtk.ResponseType.OK) {
@@ -566,7 +484,7 @@ class PanelManagerPreferences extends Gtk.Notebook {
             }
             dialog.destroy();
         });
-        dialog.show_all();
+        dialog.show();
     }
 });
 
@@ -574,6 +492,6 @@ function init() {
     ExtensionUtils.initTranslations();
 }
 
-function buildPrefsWidget() {
-    return new PanelManagerPreferences();
+function fillPreferencesWindow(window) {
+    window.add(new PanelManagerPreferences());
 }
